@@ -1,13 +1,13 @@
 from __future__ import print_function
 
-doc = r"""
+doc = r"""\
 The ``decorator`` module
-=============================================================
+------------------------
 
 :Author: Michele Simionato
 :E-mail: michele.simionato@gmail.com
 :Version: $VERSION ($DATE)
-:Supports: Python 2.6, 2.7, 3.0, 3.1, 3.2, 3.3, 3.4, 3.5
+:Supports: Python 2.6, 2.7, 3.0, 3.1, 3.2, 3.3, 3.4, 3.5, 3.6
 :Download page: http://pypi.python.org/pypi/decorator/$VERSION
 :Installation: ``pip install decorator``
 :License: BSD license
@@ -33,12 +33,12 @@ trivial to distribute the module as an universal wheel_ since 2to3 is no more
 required. Since Python 2.5 has been released 9 years ago, I felt that
 it was reasonable to drop the support for it. If you need to support
 ancient versions of Python, stick with the decorator module version
-3.4.2.  The current version supports all Python releases from 2.6 up to 3.5.
+3.4.2.  The current version supports all Python releases from 2.6 up to 3.6.
 
 .. _wheel: http://pythonwheels.com/
 
-What's New
----------------------
+What's New in version 4
+-----------------------
 
 - **New documentation**
   There is now a single manual for all Python versions, so I took the
@@ -57,7 +57,7 @@ What's New
   ``decorator(caller, func)``. The old functionality is now deprecated
   and no longer documented, but still available for now.
 
-- **New experimental feature**
+- **Multiple dispatch**
   The decorator module now includes an implementation of generic
   functions (sometimes called "multiple dispatch functions").
   The API is designed to mimic ``functools.singledispatch`` (added
@@ -70,6 +70,11 @@ What's New
   (less then 100 lines), so you can extract it for your own use.
   Take it as food for thought.
 
+- **Python 3.5 coroutines**
+  From version 4.1 it is possible to decorate coroutines, i.e. functions
+  defined with the `async def` syntax, and to mantain the
+  `inspect.iscoroutinefunction` check working for the decorated function.
+
 Usefulness of decorators
 ------------------------------------------------
 
@@ -156,7 +161,7 @@ Here I used the functools.update_wrapper_ utility, which was added
 in Python 2.5 to simplify the writing of decorators.
 (Previously, you needed to manually copy the function attributes
 ``__name__``, ``__doc__``, ``__module__``, and ``__dict__``
-to the decorated function by hand.)
+to the decorated function by hand).
 
 Here is an example of usage:
 
@@ -307,11 +312,11 @@ $FUNCTION_ANNOTATIONS
 ``decorator.decorator``
 ---------------------------------------------
 
-It can become tedious to write a caller function (like the above  
+It can become tedious to write a caller function (like the above
 ``_trace`` example) and then a trivial wrapper
-(``def trace(f): return decorate(f, _trace)``) every time. 
-Not to worry!  The ``decorator`` module provides an easy shortcut 
-to convert the caller function into a signature-preserving decorator. 
+(``def trace(f): return decorate(f, _trace)``) every time.
+Not to worry!  The ``decorator`` module provides an easy shortcut
+to convert the caller function into a signature-preserving decorator.
 
 It is the ``decorator`` function:
 
@@ -445,7 +450,7 @@ factory. For instance, if you write this...
 
 
 ...then ``before_after`` is a factory function that returns
-``GeneratorContextManager`` objects, which provide the 
+``GeneratorContextManager`` objects, which provide the
 use of the ``with`` statement:
 
 .. code-block:: python
@@ -474,9 +479,9 @@ a ``__call__`` method, so that they can be used as decorators, like so:
  AFTER
 
 The ``ba`` decorator basically inserts a ``with ba:`` block
-inside the function. 
+inside the function.
 
-However, there two issues:
+However, there are two issues:
 
 1. ``GeneratorContextManager`` objects are only callable in Python 3.2,
    so the previous example breaks in older versions of Python.
@@ -596,7 +601,7 @@ decorated functions. In IPython, this means that the usual ``??`` trick
 will give you the (right on the spot) message ``Dynamically generated
 function. No source code available``.
 
-In the past, I have considered this acceptable, since ``inspect.getsource``
+In the past, I considered this acceptable, since ``inspect.getsource``
 does not really work with "regular" decorators. In those cases,
 ``inspect.getsource`` gives you the wrapper source code, which is probably
 not what you want:
@@ -613,8 +618,7 @@ $$example
  <BLANKLINE>
 
 (See bug report 1764286_ for an explanation of what is happening).
-Unfortunately the bug still exists in all versions of Python, except
-Python 3.5.
+Unfortunately the bug still exists in all versions of Python < 3.5.
 
 However, there is a workaround. The decorated function has the ``__wrapped__``
 attribute, pointing to the original function. The simplest way to get the
@@ -640,16 +644,16 @@ Sometimes on the net you find some cool decorator that you would
 like to include in your code. However, more often than not, the cool
 decorator is not signature-preserving. What you need is an easy way to
 upgrade third party decorators to signature-preserving decorators...
-*without* having to rewrite them in terms of ``decorator``. 
+*without* having to rewrite them in terms of ``decorator``.
 
 You can use a ``FunctionMaker`` to implement that functionality as follows:
 
 $$decorator_apply
 
-``decorator_apply`` sets the generated function's ``__wrapped__`` attribute 
-to the original function, so you can get the right source code. 
-If you are using a Python later than 3.2, you should also set the 
-``__qualname__`` attribute to preserve the qualified name of the original 
+``decorator_apply`` sets the generated function's ``__wrapped__`` attribute
+to the original function, so you can get the right source code.
+If you are using a Python later than 3.2, you should also set the
+``__qualname__`` attribute to preserve the qualified name of the original
 function.
 
 Notice that I am not providing this functionality in the ``decorator``
@@ -658,8 +662,8 @@ of adding another level of indirection. However, practicality
 beats purity, so you can add ``decorator_apply`` to your toolbox and
 use it if you need to.
 
-To give a good example for ``decorator_apply``, I will show a pretty slick 
-decorator that converts a tail-recursive function into an iterative function. 
+To give a good example for ``decorator_apply``, I will show a pretty slick
+decorator that converts a tail-recursive function into an iterative function.
 I have shamelessly stolen the core concept from Kay Schluehr's recipe
 in the Python Cookbook,
 http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/496691.
@@ -681,10 +685,10 @@ $$factorial
  24
 
 This decorator is pretty impressive, and should give you some food for
-thought! ;) 
+thought! ;)
 
-Notice that there is no recursion limit now; you can easily compute 
-``factorial(1001)`` (or larger) without filling the stack frame. 
+Notice that there is no recursion limit now; you can easily compute
+``factorial(1001)`` (or larger) without filling the stack frame.
 
 Notice also that the decorator will *not* work on functions which
 are not tail recursive, such as the following:
@@ -697,6 +701,61 @@ following:
 - returns a value without making a recursive call; or,
 - returns directly the result of a recursive call.
 
+Python 3.5 coroutines
+-----------------------
+
+I am personally not using Python 3.5 coroutines yet, because at work we are
+still maintaining compatibility with Python 2.7. However, some users requested
+support for coroutines and since version 4.1 the decorator module has it.
+You should consider the support experimental and kindly report issues if
+you find any.
+
+Here I will give a single example of usage. Suppose you want to log the moment
+a coroutine starts and the moment it stops for debugging purposes. You could
+write code like the following:
+
+.. code-block:: python
+
+ import time
+ import logging
+ from asyncio import get_event_loop, sleep, wait
+ from decorator import decorator
+
+ @decorator
+ async def log_start_stop(coro, *args, **kwargs):
+     logging.info('Starting %s%s', coro.__name__, args)
+     t0 = time.time()
+     await coro(*args, **kwargs)
+     dt = time.time() - t0
+     logging.info('Ending %s%s after %d seconds', coro.__name__, args, dt)
+
+ @log_start_stop
+ async def make_task(n):
+     for i in range(n):
+         await sleep(1)
+
+ if __name__ == '__main__':
+     logging.basicConfig(level=logging.INFO)
+     tasks = [make_task(3), make_task(2), make_task(1)]
+     get_event_loop().run_until_complete(wait(tasks))
+
+and you will get at output like this::
+
+ INFO:root:Starting make_task(1,)
+ INFO:root:Starting make_task(3,)
+ INFO:root:Starting make_task(2,)
+ INFO:root:Ending make_task(1,) after 1 seconds
+ INFO:root:Ending make_task(2,) after 2 seconds
+ INFO:root:Ending make_task(3,) after 3 seconds
+
+This may be handy if you have trouble understanding what it going on
+with a particularly complex chain of coroutines. With a single line you
+can decorate the troubling coroutine function, understand what happens, fix the
+issue and then remove the decorator (or keep it if continuous monitoring
+of the coroutines makes sense). Notice that
+`inspect.iscoroutinefunction(make_task)`
+will return then right answer (i.e. `True`).
+
 Multiple dispatch
 -------------------------------------------
 
@@ -725,10 +784,12 @@ $$XMLWriter
 
 Here, you want to dispatch on the *second* argument; the first is already
 taken by ``self``. The ``dispatch_on`` decorator factory allows you to specify
-the dispatch argument simplpy by passing its name as a string. (Note
-that if you mispell the name you will get an error.)
+the dispatch argument simply by passing its name as a string. (Note
+that if you misspell the name you will get an error.)
 
-The decorated function decorated is turned into a generic function,
+The decorated function `write` is turned into a generic function (
+`write` is a function at the idea it is decorated; it will be turned
+into a method later, at class instantiation time),
 and it is called if there are no more specialized implementations.
 
 Usually, default functions should raise a ``NotImplementedError``, thus
@@ -972,7 +1033,7 @@ not use any cache, whereas the ``singledispatch`` implementation does.
 Caveats and limitations
 -------------------------------------------
 
-One thing you should be aware of is the performance penalty of decorators.
+One thing you should be aware of, is the performance penalty of decorators.
 The worse case is shown by the following example:
 
 .. code-block:: bash
@@ -1002,13 +1063,13 @@ plain function is five times slower::
  1000000 loops, best of 3: 1.39 usec per loop
  1000000 loops, best of 3: 0.278 usec per loop
 
-Of course, a real life function probably does something more useful 
-than ``f`` here, so the real life performance penalty *could* be negligible.  
-As always, the only way to know if there is a penalty in your specific use case 
-is to measure it.
+Of course, a real life function probably does something more useful
+than the function ``f`` here, so the real life performance penalty
+*could* be negligible.  As always, the only way to know if there is a
+penalty in your specific use case is to measure it.
 
 More importantly, you should be aware that decorators will make your
-tracebacks longer and more difficult to understand. 
+tracebacks longer and more difficult to understand.
 
 Consider this example:
 
@@ -1018,7 +1079,7 @@ Consider this example:
  ... def f():
  ...     1/0
 
-Calling ``f()`` gives you a ``ZeroDivisionError``. 
+Calling ``f()`` gives you a ``ZeroDivisionError``.
 But since the function is decorated, the traceback is longer:
 
 .. code-block:: python
@@ -1036,9 +1097,9 @@ But since the function is decorated, the traceback is longer:
 You see here the inner call to the decorator ``trace``, which calls
 ``f(*args, **kw)``, and a reference to  ``File "<string>", line 2, in f``.
 
-This latter reference is due to the fact that internally, the decorator
+This latter reference is due to the fact that, internally, the decorator
 module uses ``exec`` to generate the decorated function. Notice that
-``exec`` is *not* responsibile for the performance penalty, since is the
+``exec`` is *not* responsible for the performance penalty, since is the
 called *only once* (at function decoration time); it is *not* called
 each time the decorated function is called.
 
@@ -1056,22 +1117,28 @@ been made: you can decorate a function with
 ``func_tools.update_wrapper``, and ``pydoc`` will see the correct
 signature. Unfortunately, the function will still have an incorrect
 signature internally, as you can see by using
-``inspect.getfullargspec``; so, all documentation tools using the
-function (which has rightly been deprecated) will see the wrong
-signature.
+``inspect.getfullargspec``; so, all documentation tools using
+``inspect.getfullargspec`` - which has been rightly deprecated -
+will see the wrong signature.
 
 .. _362: http://www.python.org/dev/peps/pep-0362
 
 In the present implementation, decorators generated by ``decorator``
 can only be used on user-defined Python functions or methods.
-They cannot be used on generic callable objects or built-in functions, 
-due to limitations of the standard library's ``inspect`` module, especially 
-for Python 2. (In Python 3.5, many such limitations have been removed.)
+They cannot be used on generic callable objects or built-in functions,
+due to limitations of the standard library's ``inspect`` module, especially
+for Python 2. In Python 3.5, many such limitations have been removed, but
+I still think that it is cleaner and safer to decorate only
+functions. If you want to decorate things like classmethods/staticmethods
+and general callables - which I will never support in the decorator module -
+I suggest you to look at the wrapt_ project by Graeme Dumpleton.
+
+.. _wrapt: https://wrapt.readthedocs.io/en/latest/
 
 There is a strange quirk when decorating functions with keyword
 arguments, if one of the arguments has the same name used in the
 caller function for the first argument. The quirk was reported by
-David Goldstein. 
+David Goldstein.
 
 Here is an example where it is manifest:
 
@@ -1109,10 +1176,13 @@ or to change the implementation like so:
 
 This avoids the need to name the first argument, so the problem
 simply disappears. This is a technique that you should keep in mind
-when writing decorators for functions with keyword arguments.
+when writing decorators for functions with keyword arguments. Also,
+notice that lately I have come to believe that decorating functions with
+keyword arguments is not such a good idea, and you may want not to do
+that.
 
-On a similar note, there is a restriction on argument names. For instance, 
-if name an argument ``_call_`` or ``_func_``, you will get a ``NameError``:
+On a similar note, there is a restriction on argument names. For instance,
+if you name an argument ``_call_`` or ``_func_``, you will get a ``NameError``:
 
 .. code-block:: python
 
@@ -1146,10 +1216,10 @@ a (shallow) copy of the original function dictionary:
 .. _docutils: http://docutils.sourceforge.net/
 .. _pygments: http://pygments.org/
 
-LICENSE
+LICENSE (2-clause BSD)
 ---------------------------------------------
 
-Copyright (c) 2005-2016, Michele Simionato
+Copyright (c) 2005-2017, Michele Simionato
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -1225,11 +1295,11 @@ You can check that the ``__annotations__`` dictionary is preserved:
   >>> f.__annotations__ is f.__wrapped__.__annotations__
   True
 
-Here ``f.__wrapped__`` is the original undecorated function. 
-This attribute exists for consistency with the behavior of 
+Here ``f.__wrapped__`` is the original undecorated function.
+This attribute exists for consistency with the behavior of
 ``functools.update_wrapper``.
 
-Another attribute copied from the original function is ``__qualname__``, 
+Another attribute copied from the original function is ``__qualname__``,
 the qualified name. This attribute was introduced in Python 3.3.
 """
 
@@ -1341,7 +1411,7 @@ def memoize(f):
     """
     A simple memoize implementation. It works by adding a .cache dictionary
     to the decorated function. The cache will grow indefinitely, so it is
-    your responsability to clear it, if needed.
+    your responsibility to clear it, if needed.
     """
     f.cache = {}
     return decorate(f, _memoize)
@@ -1481,6 +1551,7 @@ def a_test_for_pylons():
     'The good old factorial'
     """
 
+
 if sys.version >= '3':  # tests for signatures specific to Python 3
 
     def test_kwonlydefaults():
@@ -1529,6 +1600,7 @@ def before_after(before, after):
     yield
     print(after)
 
+
 ba = before_after('BEFORE', 'AFTER')  # ContextManager instance
 
 
@@ -1636,6 +1708,8 @@ def get_length_set(obj):
 
 class C(object):
     "Registered as Sized and Iterable"
+
+
 collections.Sized.register(C)
 collections.Iterable.register(C)
 
