@@ -9,7 +9,7 @@ import collections
 from decorator import dispatch_on, contextmanager, decorator
 try:
     from . import documentation as doc
-except (SystemError, ValueError):
+except:
     import documentation as doc
 
 
@@ -23,6 +23,24 @@ def assertRaises(etype):
     else:
         raise Exception('Expected %s' % etype.__name__)
 
+if sys.version >= '3.5':
+    exec('''from asyncio import get_event_loop
+
+@decorator
+async def before_after(coro, *args, **kwargs):
+    return "<before>" + (await coro(*args, **kwargs)) + "<after>"
+
+
+class CoroutineTestCase(unittest.TestCase):
+    def test(self):
+        @before_after
+        async def coro(x):
+           return x
+        self.assertTrue(inspect.iscoroutinefunction(coro))
+        out = get_event_loop().run_until_complete(coro('x'))
+        self.assertEqual(out, '<before>x<after>')
+''')
+
 
 class DocumentationTestCase(unittest.TestCase):
     def test(self):
@@ -92,6 +110,7 @@ class ExtraTestCase(unittest.TestCase):
         # there is no confusion when passing args as a keyword argument
         self.assertEqual(func(args='a'), {'args': 'a'})
 
+
 # ################### test dispatch_on ############################# #
 # adapted from test_functools in Python 3.5
 singledispatch = dispatch_on('obj')
@@ -410,5 +429,6 @@ class TestSingleDispatch(unittest.TestCase):
         with assertRaises(RuntimeError):
             h(u)
 
+
 if __name__ == '__main__':
     unittest.main()
